### R Script for "COPS in Action: Exploring Structure in the Usage of the Modular Approach to Therapy for Children (MATCH)" submitted to Psych
## License: GPL-2


###################################################
### Packages needed
###################################################

## For calculations
library(dbscan)
library(cordillera)
library(smacof)
library(cops)

## Extra 
library(colorspace)
library(ggplot2)
library(ggrepel) #for labelling
library(ggthemes) #for theme
library(gridExtra)
library(ggplotify)


###################################################
### Distance Matrix
###################################################
distM <- cops::matchphi


###################################################
### Figure 1
###################################################
idf <- reshape::melt(distM)
colnames(idf) <- c("x", "y", 'distance')
ggplot(idf,aes(x,y,fill=distance))+geom_tile()+guides(fill = guide_colourbar(barwidth = 1,barheight = 20))+theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1),axis.title.x=element_blank(),axis.title.y=element_blank()) +scale_fill_gradientn(colors=sequential_hcl(100,palette="Teal"))


###################################################
### OPTICS calculations 
###################################################
o1 <- dbscan::optics(as.dist(distM),minPts=3,eps=10)
c1 <- cordillera::cordillera(as.dist(distM),minpts=3,eps=10,q=2)


###################################################
### Finding clusters with xi-strategy
###################################################
cls <- dbscan::extractXi(o1,0.039)

###################################################
### An OPTICS plot that we can adjust
###################################################

##We needed to hack this because plot.optics produces a par() side effect... 
plot.opticsH<-function (x, cluster = TRUE, predecessor = FALSE, ...) 
{
    if (inherits(x$cluster, "xics") || all(c("start", "end", 
        "cluster_id") %in% names(x$clusters_xi))) {
        hclusters <- x$clusters_xi[order(x$clusters_xi$end - 
            x$clusters_xi$start), ]
        #def.par <- par(no.readonly = TRUE)
        #par(mar = c(2, 4, 4, 2) + 0.1, omd = c(0, 1, 0.15, 1))
        y_max <- max(x$reachdist[which(x$reachdist != Inf)])
        y_increments <- (y_max/0.85 * 0.15)/(nrow(hclusters) + 
            1L)
        plot(as.reachability(x), col = x$cluster[x$order] + 1L, 
            xlab = NA, xaxt = "n", yaxs = "i", ylim = c(0, y_max), 
            ...)
        i <- 1:nrow(hclusters)
        segments(x0 = hclusters$start[i], y0 = -(y_increments * 
            i), x1 = hclusters$end[i], col = hclusters$cluster_id[i] + 
            1L, lwd = 2, xpd = NA)
        #par(def.par)
    }
    else if (is.numeric(x$cluster) && !is.null(x$eps_cl)) {
        plot(as.reachability(x), col = x$cluster[x$order] + 1L, 
            ...)
        lines(x = c(0, length(x$cluster)), y = c(x$eps_cl, x$eps_cl), 
            col = "black", lty = 2)
    }
    else {
        plot(as.reachability(x), ...)
    }
}



###################################################
### Figure 2
###################################################
par(mfrow=c(2,1))
plot(c1)
plot.opticsH(cls,main="")
par(mfrow=c(1,1))



###################################################
### Aux Func for nice plotting with repelling labels
###################################################
plotrep <- function(object,alpha=1,col=NULL,col1=NULL,title="plot",...)
{
    if(is.null(col)) {
        col <- "black"
    }
    if(is.null(col1)) {
        col1 <- "grey35"
    }
    conf <- as.data.frame(object$conf)
    #conf <- cbind(conf,factor)
    p1 <- ggplot(conf,aes(x=D1,y=D2,label=rownames(conf)))+geom_point(size=2,alpha=alpha,color=col)+coord_fixed()+theme_few()+geom_text_repel(max.overlaps=Inf,color=col1,segment.color="grey50")+ggtitle(title)
   invisible(p1)
}

###################################################
### Parameter Setup for MDS and COPS
###################################################
## OC parameters
minpts <- 3
dmax <- 1
q <- 2

##iterations
itm <- 50000

###################################################
### Setting up v1 and v2
###################################################
## Weights
cw <- 0.025
sw <- 0.975


###################################################
### Standard Ratio MDS
###################################################
## Standard MDS
set.seed(123)
MDSM0 <- cops::copstressMin(distM,type="ratio", #distance object and MDS type       
                     dmax=dmax,minpts=minpts,q=q, #metaparameters
                     stressweight=1,cordweight=0, #v1 and v2
                     itmax=itm) 
MDSM0


###################################################
### Figure 3
###################################################
prat0 <- plotrep(MDSM0,title="Ratio MDS")
prat0rp <- as.ggplot(~plot(MDSM0,plot.type="reachplot"))
grid.arrange(prat0,prat0rp)



###################################################
### Ratio COPS-C
###################################################
## Ratio COPS-C
MDSM <- cops::copstressMin(distM,type="ratio",   
                    dmax=dmax,minpts=minpts,q=q, 
                    stressweight=sw,cordweight=cw, 
                    itmax=itm) 
MDSM


###################################################
### Figure 4
###################################################
prat <- plotrep(MDSM,title="Ratio COPS-C")
pratrp <- as.ggplot(~plot(MDSM,plot.type="reachplot"))
grid.arrange(prat,pratrp)


###################################################
### Interval COPS-C
###################################################
## Interval COPS-C
MDSMint <- cops::copstressMin(distM,type="interval",   
                       dmax=dmax,minpts=minpts,q=q,
                       stressweight=sw,cordweight=cw, 
                       itmax=itm) 
MDSMint


###################################################
### Figure 5 
###################################################
pint <- plotrep(MDSMint,title="Interval COPS-C")
pintrp <- as.ggplot(~plot(MDSMint,plot.type="reachplot"))
grid.arrange(pint,pintrp)


###################################################
### Multiscale COPS-C 
###################################################
distM2 <- log(1+distM) 
diag(distM2) <- 0
MDSMmsc <- cops::copstressMin(distM2,type="ratio",kappa=0.1, 
                       dmax=dmax,minpts=minpts,q=q,   
                       stressweight=sw,cordweight=cw, 
                       itmax=itm)
MDSMmsc


###################################################
### Figure 7
###################################################
pmsc <- plotrep(MDSMmsc,title="Multiscale COPS-C")
pmscrp <- as.ggplot(~plot(MDSMmsc,plot.type="reachplot"))
grid.arrange(pmsc,pmscrp)



###################################################
### S-Stress COPS-C 
###################################################
## ## S-Stress COPS-C
MDSMsst <- cops::copstressMin(distM,type="ratio",kappa=2,lambda=2,
                       dmax=dmax,minpts=minpts,q=q,        
                       stressweight=sw,cordweight=cw, 
                       itmax=itm) 
MDSMsst


###################################################
### Figure 6
###################################################
psst <- plotrep(MDSMsst,title="S-Stress COPS-C")
psstrp <- as.ggplot(~plot(MDSMsst,plot.type="reachplot"))
grid.arrange(psst,psstrp)



###################################################
### P-COPS 
###################################################
## ## P-COPS (powermds loss) 
MDSMpco <- cops::pcops(distM,loss="powermds", 
                  minpts=minpts,rang=c(0,dmax), 
                  upper=c(5,5),lower=c(0.01,0.01), 
                 itmaxi=10000)
MDSMpco


###################################################
### Power Stress COPS-C with P-COPS parameters 
###################################################
## ## Power Stress COPS-C
MDSMpst <- cops::copstressMin(distM,kappa=MDSMpco$par[1],lambda=MDSMpco$par[2],
                         init=MDSMpco$fit$conf,dmax=dmax,minpts=minpts,q=q,
                         stressweight=sw,cordweight=cw,
                         itmax=itm) 
MDSMpst


###################################################
### Figure 8 
###################################################
ppst <- plotrep(MDSMpst,title="P-Stress COPS-C")
ppstrp <- as.ggplot(~plot(MDSMpst,plot.type="reachplot"))
grid.arrange(ppst,ppstrp)

###################################################
### Facet setup
###################################################
## Facets
protocol<-factor(c("GEN","GEN",rep("ANX",6),rep("TRA",2),
                rep("DEP",10),rep("CON",12))) #Based on protocols


###################################################
### Fit SVM 
###################################################
## Setup
library(e1071)
kernel <- "radial"
dat1 <- data.frame(facet = protocol, MDSMint$conf)
dat2 <- data.frame(facet = protocol, MDSMpst$conf)

##cost of constraint violation
costvec <- seq(10, 100, 2)   ## costs of constraint violations

##Fitting the SVM 
set.seed(1)
svm_mdint <- e1071::tune.svm(facet ~ D1 + D2, data = dat1, kernel = kernel, 
                      cross = 10, cost = costvec)$best.model
svm_mdpst <- e1071::tune.svm(facet ~ D1 + D2, data = dat2, kernel = kernel, 
                      cross = 10, cost = costvec)$best.model


###################################################
### Figure 9
###################################################
par(mfrow=c(2,1))
smacof::svm_mdsplot(MDSMint, svm_mdint, protocol, main = "Interval SVM-COPS Facets: 
            MATCH Protocols",legend2=FALSE)  
smacof::svm_mdsplot(MDSMpst, svm_mdpst, protocol, main = "P-Stress SVM-COPS Facets: 
            MATCH Protocols",legend2=FALSE)  
par(mfrow=c(1,1))


###################################################
### Setup ESPC and Protocol
###################################################
espcat <- data.frame(module=colnames(matchphi),
                     espc=c("RelBuild","FutPlan",rep("RelBuild",3),"TTO",
                            "FutPlan","RepThou","SolvProb","TTO","RelBuild",
                            "RelBuild","SolvProb","TTO","FeelCalm",
                            "FeelCalm","TTO","RepThou","RepThou","FutPlan",
                            "RelBuild","RelBuild",rep("IncMot",6),"SolvProb",
                            "IncMot","FutPlan","FutPlan"),
                     protocol=protocol)
thespc <- factor(paste(espcat$protocol, espcat$espc, sep="-")) #combined category


###################################################
### Aux func for plotting with repelled labels 
###################################################
plotrep2 <- function(object,alpha=1,title="plot",factvar,...)
{
    conf <- as.data.frame(object$conf)
    conf <- cbind(conf,fac=as.factor(factvar))
    cols <- rainbow_hcl(length(levels(factvar)))
    p1 <- ggplot(conf,aes(x=D1,y=D2,label=rownames(conf),color=fac))+geom_point(size=2,alpha=alpha)+coord_fixed()+theme_few()+ggtitle(title)+geom_text_repel(max.overlaps=Inf,segment.color="grey50")
  invisible(p1)
}


###################################################
### Figure 10
###################################################
frat <- plotrep2(MDSM,factvar=thespc,title="Ratio COPS-C")
fint <- plotrep2(MDSMint,factvar=thespc,title="Interval COPS-C")
grid.arrange(frat,fint)


